# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from typing import Final

from opentelemetry.metrics import Histogram, Meter

MCP_CLIENT_OPERATION_DURATION: Final = "mcp.client.operation.duration"
"""
The duration of the MCP request or notification as observed on the sender from the time it was sent until the response or ack is received
Instrument: histogram
Unit: s
"""


def create_mcp_client_operation_duration(meter: Meter) -> Histogram:
    """The duration of the MCP request or notification as observed on the sender from the time it was sent until the response or ack is received"""
    return meter.create_histogram(
        name=MCP_CLIENT_OPERATION_DURATION,
        description="The duration of the MCP request or notification as observed on the sender from the time it was sent until the response or ack is received.",
        unit="s",
    )


MCP_CLIENT_SESSION_DURATION: Final = "mcp.client.session.duration"
"""
The duration of the MCP session as observed on the MCP client
Instrument: histogram
Unit: s
"""


def create_mcp_client_session_duration(meter: Meter) -> Histogram:
    """The duration of the MCP session as observed on the MCP client"""
    return meter.create_histogram(
        name=MCP_CLIENT_SESSION_DURATION,
        description="The duration of the MCP session as observed on the MCP client.",
        unit="s",
    )


MCP_SERVER_OPERATION_DURATION: Final = "mcp.server.operation.duration"
"""
MCP request or notification duration as observed on the receiver from the time it was received until the result or ack is sent
Instrument: histogram
Unit: s
"""


def create_mcp_server_operation_duration(meter: Meter) -> Histogram:
    """MCP request or notification duration as observed on the receiver from the time it was received until the result or ack is sent"""
    return meter.create_histogram(
        name=MCP_SERVER_OPERATION_DURATION,
        description="MCP request or notification duration as observed on the receiver from the time it was received until the result or ack is sent.",
        unit="s",
    )


MCP_SERVER_SESSION_DURATION: Final = "mcp.server.session.duration"
"""
The duration of the MCP session as observed on the MCP server
Instrument: histogram
Unit: s
"""


def create_mcp_server_session_duration(meter: Meter) -> Histogram:
    """The duration of the MCP session as observed on the MCP server"""
    return meter.create_histogram(
        name=MCP_SERVER_SESSION_DURATION,
        description="The duration of the MCP session as observed on the MCP server.",
        unit="s",
    )



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Final

THREAD_ID: Final = "thread.id"
"""
Current "managed" thread ID (as opposed to OS thread ID).
Note: Examples of where the value can be extracted from:

| Language or platform | Source |
| --- | --- |
| JVM | `Thread.currentThread().threadId()` |
| .NET | `Thread.CurrentThread.ManagedThreadId` |
| Python | `threading.current_thread().ident` |
| Ruby | `Thread.current.object_id` |
| C++ | `std::this_thread::get_id()` |
| Erlang | `erlang:self()` |.
"""

THREAD_NAME: Final = "thread.name"
"""
Current thread name.
Note: Examples of where the value can be extracted from:

| Language or platform | Source |
| --- | --- |
| JVM | `Thread.currentThread().getName()` |
| .NET | `Thread.CurrentThread.Name` |
| Python | `threading.current_thread().name` |
| Ruby | `Thread.current.name` |
| Erlang | `erlang:process_info(self(), registered_name)` |.
"""



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enum import Enum
from typing import Final

OPENAI_API_TYPE: Final = "openai.api.type"
"""
The type of OpenAI API being used.
"""

OPENAI_REQUEST_SERVICE_TIER: Final = "openai.request.service_tier"
"""
The service tier requested. May be a specific tier, default, or auto.
"""

OPENAI_RESPONSE_SERVICE_TIER: Final = "openai.response.service_tier"
"""
The service tier used for the response.
"""

OPENAI_RESPONSE_SYSTEM_FINGERPRINT: Final = (
    "openai.response.system_fingerprint"
)
"""
A fingerprint to track any eventual change in the Generative AI environment.
"""


class OpenaiApiTypeValues(Enum):
    CHAT_COMPLETIONS = "chat_completions"
    """The OpenAI [Chat Completions API](https://developers.openai.com/api/reference/chat-completions/overview)."""
    RESPONSES = "responses"
    """The OpenAI [Responses API](https://developers.openai.com/api/reference/responses/overview)."""


class OpenaiRequestServiceTierValues(Enum):
    AUTO = "auto"
    """The system will utilize scale tier credits until they are exhausted."""
    DEFAULT = "default"
    """The system will utilize the default scale tier."""



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enum import Enum
from typing import Final

MCP_METHOD_NAME: Final = "mcp.method.name"
"""
The name of the request or notification method.
"""

MCP_PROTOCOL_VERSION: Final = "mcp.protocol.version"
"""
The [version](https://modelcontextprotocol.io/specification/versioning) of the Model Context Protocol used.
"""

MCP_RESOURCE_URI: Final = "mcp.resource.uri"
"""
The value of the resource uri.
Note: This is a URI of the resource provided in the following requests or notifications: `resources/read`, `resources/subscribe`, `resources/unsubscribe`, or `notifications/resources/updated`.
"""

MCP_SESSION_ID: Final = "mcp.session.id"
"""
Identifies [MCP session](https://modelcontextprotocol.io/specification/2025-06-18/basic/transports#session-management).
"""


class McpMethodNameValues(Enum):
    NOTIFICATIONS_CANCELLED = "notifications/cancelled"
    """Notification cancelling a previously-issued request."""
    INITIALIZE = "initialize"
    """Request to initialize the MCP client."""
    NOTIFICATIONS_INITIALIZED = "notifications/initialized"
    """Notification indicating that the MCP client has been initialized."""
    NOTIFICATIONS_PROGRESS = "notifications/progress"
    """Notification indicating the progress for a long-running operation."""
    PING = "ping"
    """Request to check that the other party is still alive."""
    RESOURCES_LIST = "resources/list"
    """Request to list resources available on server."""
    RESOURCES_TEMPLATES_LIST = "resources/templates/list"
    """Request to list resource templates available on server."""
    RESOURCES_READ = "resources/read"
    """Request to read a resource."""
    NOTIFICATIONS_RESOURCES_LIST_CHANGED = (
        "notifications/resources/list_changed"
    )
    """Notification indicating that the list of resources has changed."""
    RESOURCES_SUBSCRIBE = "resources/subscribe"
    """Request to subscribe to a resource."""
    RESOURCES_UNSUBSCRIBE = "resources/unsubscribe"
    """Request to unsubscribe from resource updates."""
    NOTIFICATIONS_RESOURCES_UPDATED = "notifications/resources/updated"
    """Notification indicating that a resource has been updated."""
    PROMPTS_LIST = "prompts/list"
    """Request to list prompts available on server."""
    PROMPTS_GET = "prompts/get"
    """Request to get a prompt."""
    NOTIFICATIONS_PROMPTS_LIST_CHANGED = "notifications/prompts/list_changed"
    """Notification indicating that the list of prompts has changed."""
    TOOLS_LIST = "tools/list"
    """Request to list tools available on server."""
    TOOLS_CALL = "tools/call"
    """Request to call a tool."""
    NOTIFICATIONS_TOOLS_LIST_CHANGED = "notifications/tools/list_changed"
    """Notification indicating that the list of tools has changed."""
    LOGGING_SET_LEVEL = "logging/setLevel"
    """Request to set the logging level."""
    NOTIFICATIONS_MESSAGE = "notifications/message"
    """Notification indicating that a message has been received."""
    SAMPLING_CREATE_MESSAGE = "sampling/createMessage"
    """Request to create a sampling message."""
    COMPLETION_COMPLETE = "completion/complete"
    """Request to complete a prompt."""
    ROOTS_LIST = "roots/list"
    """Request to list roots available on server."""
    NOTIFICATIONS_ROOTS_LIST_CHANGED = "notifications/roots/list_changed"
    """Notification indicating that the list of roots has changed."""
    ELICITATION_CREATE = "elicitation/create"
    """Request from the server to elicit additional information from the user via the client."""



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Final

EXCEPTION_ESCAPED: Final = "exception.escaped"
"""
Deprecated: It's no longer recommended to record exceptions that are handled and do not escape the scope of a span.
"""

EXCEPTION_MESSAGE: Final = "exception.message"
"""
The exception message.
Note: > [!WARNING]
>
> This attribute may contain sensitive information.
"""

EXCEPTION_STACKTRACE: Final = "exception.stacktrace"
"""
A stacktrace as a string in the natural representation for the language runtime. The representation is to be determined and documented by each language SIG.
"""

EXCEPTION_TYPE: Final = "exception.type"
"""
The type of the exception (its fully-qualified class name, if applicable). The dynamic type of the exception should be preferred over the static type in languages that support it.
"""



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enum import Enum
from typing import Final

from typing_extensions import deprecated

ERROR_MESSAGE: Final = "error.message"
"""
Deprecated: Use domain-specific error message attribute. For example, use `feature_flag.error.message` for feature flag errors.
"""

ERROR_TYPE: Final = "error.type"
"""
Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.error_attributes.ERROR_TYPE`.
"""


@deprecated(
    "Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.error_attributes.ErrorTypeValues`."
)
class ErrorTypeValues(Enum):
    OTHER = "_OTHER"
    """Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.error_attributes.ErrorTypeValues.OTHER`."""



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enum import Enum
from typing import Final

USER_AGENT_NAME: Final = "user_agent.name"
"""
Name of the user-agent extracted from original. Usually refers to the browser's name.
Note: [Example](https://uaparser.dev/#demo) of extracting browser's name from original string. In the case of using a user-agent for non-browser products, such as microservices with multiple names/versions inside the `user_agent.original`, the most significant name SHOULD be selected. In such a scenario it should align with `user_agent.version`.
"""

USER_AGENT_ORIGINAL: Final = "user_agent.original"
"""
Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.user_agent_attributes.USER_AGENT_ORIGINAL`.
"""

USER_AGENT_OS_NAME: Final = "user_agent.os.name"
"""
Human readable operating system name.
Note: For mapping user agent strings to OS names, libraries such as [ua-parser](https://github.com/ua-parser) can be utilized.
"""

USER_AGENT_OS_VERSION: Final = "user_agent.os.version"
"""
The version string of the operating system as defined in [Version Attributes](/docs/resource/README.md#version-attributes).
Note: For mapping user agent strings to OS versions, libraries such as [ua-parser](https://github.com/ua-parser) can be utilized.
"""

USER_AGENT_SYNTHETIC_TYPE: Final = "user_agent.synthetic.type"
"""
Specifies the category of synthetic traffic, such as tests or bots.
Note: This attribute MAY be derived from the contents of the `user_agent.original` attribute. Components that populate the attribute are responsible for determining what they consider to be synthetic bot or test traffic. This attribute can either be set for self-identification purposes, or on telemetry detected to be generated as a result of a synthetic request. This attribute is useful for distinguishing between genuine client traffic and synthetic traffic generated by bots or tests.
"""

USER_AGENT_VERSION: Final = "user_agent.version"
"""
Version of the user-agent extracted from original. Usually refers to the browser's version.
Note: [Example](https://uaparser.dev/#demo) of extracting browser's version from original string. In the case of using a user-agent for non-browser products, such as microservices with multiple names/versions inside the `user_agent.original`, the most significant version SHOULD be selected. In such a scenario it should align with `user_agent.name`.
"""


class UserAgentSyntheticTypeValues(Enum):
    BOT = "bot"
    """Bot source."""
    TEST = "test"
    """Synthetic test source."""



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Final

JSONRPC_PROTOCOL_VERSION: Final = "jsonrpc.protocol.version"
"""
Protocol version, as specified in the `jsonrpc` property of the request and its corresponding response.
"""

JSONRPC_REQUEST_ID: Final = "jsonrpc.request.id"
"""
A string representation of the `id` property of the request and its corresponding response.
Note: Under the [JSON-RPC specification](https://www.jsonrpc.org/specification), the `id` property may be a string, number, null, or omitted entirely. When omitted, the request is treated as a notification. Using `null` is not equivalent to omitting the `id`, but it is discouraged.
Instrumentations SHOULD NOT capture this attribute when the `id` is `null` or omitted.
"""



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Final

ONC_RPC_PROCEDURE_NAME: Final = "onc_rpc.procedure.name"
"""
ONC/Sun RPC procedure name.
"""

ONC_RPC_PROCEDURE_NUMBER: Final = "onc_rpc.procedure.number"
"""
ONC/Sun RPC procedure number.
"""

ONC_RPC_PROGRAM_NAME: Final = "onc_rpc.program.name"
"""
ONC/Sun RPC program name.
"""

ONC_RPC_VERSION: Final = "onc_rpc.version"
"""
ONC/Sun RPC program version.
"""
